@@ -26,7 +26,7 @@
 #include <sys/lock.h>
 #include "local.h"
 
-#ifdef _REENT_SMALL
+#if defined(_REENT_SMALL) && !defined(_REENT_GLOBAL_STDIO_STREAMS)
 const struct __sFILE_fake __sf_fake_stdin =
     {_NULL, 0, 0, 0, 0, {_NULL, 0}, 0, _NULL};
 const struct __sFILE_fake __sf_fake_stdout =
@@ -35,12 +35,18 @@ const struct __sFILE_fake __sf_fake_stderr =
     {_NULL, 0, 0, 0, 0, {_NULL, 0}, 0, _NULL};
 #endif
 
-static _VOID
-_DEFUN(std, (ptr, flags, file, data),
-            FILE *ptr _AND
-            int flags _AND
-            int file  _AND
-            struct _reent *data)
+#ifdef _REENT_GLOBAL_STDIO_STREAMS
+__FILE __sf[3];
+#endif
+
+#if (defined (__OPTIMIZE_SIZE__) || defined (PREFER_SIZE_OVER_SPEED))
+_NOINLINE_STATIC void
+#else
+static void
+#endif
+std (FILE *ptr,
+            int flags,
+            int file)
 {
   ptr->_p = 0;
   ptr->_r = 0;
@@ -67,7 +73,7 @@ _DEFUN(std, (ptr, flags, file, data),
 #else /* _STDIO_CLOSE_STD_STREAMS */
   ptr->_close = NULL;
 #endif /* _STDIO_CLOSE_STD_STREAMS */
-#if !defined(__SINGLE_THREAD__) && !defined(_REENT_SMALL)
+#if !defined(__SINGLE_THREAD__) && !(defined(_REENT_SMALL) && !defined(_REENT_GLOBAL_STDIO_STREAMS))
   __lock_init_recursive (ptr->_lock);
   /*
    * #else
@@ -81,14 +87,43 @@ _DEFUN(std, (ptr, flags, file, data),
 #endif
 }
 
+static inline void
+stdin_init(FILE *ptr)
+{
+  std (ptr,  __SRD, 0);
+}
+
+static inline void
+stdout_init(FILE *ptr)
+{
+  /* On platforms that have true file system I/O, we can verify
+     whether stdout is an interactive terminal or not, as part of
+     __smakebuf on first use of the stream.  For all other platforms,
+     we will default to line buffered mode here.  Technically, POSIX
+     requires both stdin and stdout to be line-buffered, but tradition
+     leaves stdin alone on systems without fcntl.  */
+#ifdef HAVE_FCNTL
+  std (ptr, __SWR, 1);
+#else
+  std (ptr, __SWR | __SLBF, 1);
+#endif
+}
+
+static inline void
+stderr_init(FILE *ptr)
+{
+  /* POSIX requires stderr to be opened for reading and writing, even
+     when the underlying fd 2 is write-only.  */
+  std (ptr, __SRW | __SNBF, 2);
+}
+
 struct glue_with_file {
   struct _glue glue;
   FILE file;
 };
 
 struct _glue *
-_DEFUN(__sfmoreglue, (d, n),
-       struct _reent *d _AND
+__sfmoreglue (struct _reent *d,
        register int n)
 {
   struct glue_with_file *g;
@@ -109,8 +144,7 @@ _DEFUN(__sfmoreglue, (d, n),
  */
 
 FILE *
-_DEFUN(__sfp, (d),
-       struct _reent *d)
+__sfp (struct _reent *d)
 {
   FILE *fp;
   int n;
@@ -166,17 +200,38 @@ found:
  * The name `_cleanup' is, alas, fairly well known outside stdio.
  */
 
-_VOID
-_DEFUN(_cleanup_r, (ptr),
-       struct _reent *ptr)
+void
+_cleanup_r (struct _reent *ptr)
 {
-  _CAST_VOID _fwalk(ptr, fclose);
-  /* _CAST_VOID _fwalk (ptr, fflush); */	/* `cheating' */
+  int (*cleanup_func) (struct _reent *, FILE *);
+#ifdef _STDIO_BSD_SEMANTICS
+  /* BSD and Glibc systems only flush streams which have been written to
+     at exit time.  Calling flush rather than close for speed, as on
+     the aforementioned systems. */
+  cleanup_func = __sflushw_r;
+#else
+  /* Otherwise close files and flush read streams, too.
+     Note we call flush directly if "--enable-lite-exit" is in effect.  */
+#ifdef _LITE_EXIT
+  cleanup_func = _fflush_r;
+#else
+  cleanup_func = _fclose_r;
+#endif
+#endif
+#ifdef _REENT_GLOBAL_STDIO_STREAMS
+  if (ptr->_stdin != &__sf[0])
+    (*cleanup_func) (ptr, ptr->_stdin);
+  if (ptr->_stdout != &__sf[1])
+    (*cleanup_func) (ptr, ptr->_stdout);
+  if (ptr->_stderr != &__sf[2])
+    (*cleanup_func) (ptr, ptr->_stderr);
+#endif
+  (void) _fwalk_reent (ptr, cleanup_func);
 }
 
 #ifndef _REENT_ONLY
-_VOID
-_DEFUN_VOID(_cleanup)
+void
+_cleanup (void)
 {
   _cleanup_r (_GLOBAL_REENT);
 }
@@ -186,9 +241,8 @@ _DEFUN_VOID(_cleanup)
  * __sinit() is called whenever stdio's internal variables must be set up.
  */
 
-_VOID
-_DEFUN(__sinit, (s),
-       struct _reent *s)
+void
+__sinit (struct _reent *s)
 {
   __sinit_lock_acquire ();
 
@@ -203,8 +257,10 @@ _DEFUN(__sinit, (s),
 
   s->__sglue._next = NULL;
 #ifndef _REENT_SMALL
+# ifndef _REENT_GLOBAL_STDIO_STREAMS
   s->__sglue._niobs = 3;
   s->__sglue._iobs = &s->__sf[0];
+# endif /* _REENT_GLOBAL_STDIO_STREAMS */
 #else
   s->__sglue._niobs = 0;
   s->__sglue._iobs = NULL;
@@ -213,28 +269,30 @@ _DEFUN(__sinit, (s),
      __sinit if it's 0. */
   if (s == _GLOBAL_REENT)
     s->__sdidinit = 1;
+# ifndef _REENT_GLOBAL_STDIO_STREAMS
   s->_stdin = __sfp(s);
   s->_stdout = __sfp(s);
   s->_stderr = __sfp(s);
+# else /* _REENT_GLOBAL_STDIO_STREAMS */
+  s->_stdin = &__sf[0];
+  s->_stdout = &__sf[1];
+  s->_stderr = &__sf[2];
+# endif /* _REENT_GLOBAL_STDIO_STREAMS */
 #endif
 
-  std (s->_stdin,  __SRD, 0, s);
-
-  /* On platforms that have true file system I/O, we can verify
-     whether stdout is an interactive terminal or not, as part of
-     __smakebuf on first use of the stream.  For all other platforms,
-     we will default to line buffered mode here.  Technically, POSIX
-     requires both stdin and stdout to be line-buffered, but tradition
-     leaves stdin alone on systems without fcntl.  */
-#ifdef HAVE_FCNTL
-  std (s->_stdout, __SWR, 1, s);
-#else
-  std (s->_stdout, __SWR | __SLBF, 1, s);
-#endif
-
-  /* POSIX requires stderr to be opened for reading and writing, even
-     when the underlying fd 2 is write-only.  */
-  std (s->_stderr, __SRW | __SNBF, 2, s);
+#ifdef _REENT_GLOBAL_STDIO_STREAMS
+  if (__sf[0]._cookie == NULL) {
+    _GLOBAL_REENT->__sglue._niobs = 3;
+    _GLOBAL_REENT->__sglue._iobs = &__sf[0];
+    stdin_init (&__sf[0]);
+    stdout_init (&__sf[1]);
+    stderr_init (&__sf[2]);
+  }
+#else /* _REENT_GLOBAL_STDIO_STREAMS */
+  stdin_init (s->_stdin);
+  stdout_init (s->_stdout);
+  stderr_init (s->_stderr);
+#endif /* _REENT_GLOBAL_STDIO_STREAMS */
 
   s->__sdidinit = 1;
 
@@ -243,65 +301,65 @@ _DEFUN(__sinit, (s),
 
 #ifndef __SINGLE_THREAD__
 
-__LOCK_INIT_RECURSIVE(static, __sfp_lock);
-__LOCK_INIT_RECURSIVE(static, __sinit_lock);
+__LOCK_INIT_RECURSIVE(static, __sfp_recursive_mutex);
+__LOCK_INIT_RECURSIVE(static, __sinit_recursive_mutex);
 
-_VOID
-_DEFUN_VOID(__sfp_lock_acquire)
+void
+__sfp_lock_acquire (void)
 {
-  __lock_acquire_recursive (__sfp_lock);
+  __lock_acquire_recursive (__sfp_recursive_mutex);
 }
 
-_VOID
-_DEFUN_VOID(__sfp_lock_release)
+void
+__sfp_lock_release (void)
 {
-  __lock_release_recursive (__sfp_lock);
+  __lock_release_recursive (__sfp_recursive_mutex);
 }
 
-_VOID
-_DEFUN_VOID(__sinit_lock_acquire)
+void
+__sinit_lock_acquire (void)
 {
-  __lock_acquire_recursive (__sinit_lock);
+  __lock_acquire_recursive (__sinit_recursive_mutex);
 }
 
-_VOID
-_DEFUN_VOID(__sinit_lock_release)
+void
+__sinit_lock_release (void)
 {
-  __lock_release_recursive (__sinit_lock);
+  __lock_release_recursive (__sinit_recursive_mutex);
 }
 
 /* Walkable file locking routine.  */
 static int
-_DEFUN(__fp_lock, (ptr),
-       FILE * ptr)
+__fp_lock (FILE * ptr)
 {
-  _flockfile (ptr);
+  if (!(ptr->_flags2 & __SNLK))
+    _flockfile (ptr);
 
   return 0;
 }
 
 /* Walkable file unlocking routine.  */
 static int
-_DEFUN(__fp_unlock, (ptr),
-       FILE * ptr)
+__fp_unlock (FILE * ptr)
 {
-  _funlockfile (ptr);
+  if (!(ptr->_flags2 & __SNLK))
+    _funlockfile (ptr);
 
   return 0;
 }
 
-_VOID
-_DEFUN_VOID(__fp_lock_all)
+void
+__fp_lock_all (void)
 {
   __sfp_lock_acquire ();
 
-  _CAST_VOID _fwalk (_REENT, __fp_lock);
+  (void) _fwalk (_REENT, __fp_lock);
 }
 
-_VOID
-_DEFUN_VOID(__fp_unlock_all)
+void
+__fp_unlock_all (void)
 {
-  _CAST_VOID _fwalk (_REENT, __fp_unlock);
+  (void) _fwalk (_REENT, __fp_unlock);
 
   __sfp_lock_release ();
 }
