@@ -20,30 +20,16 @@
 #include <_ansi.h>
 #include <reent.h>
 #include <stdio.h>
-#ifdef _HAVE_STDC
 #include <stdarg.h>
-#else
-#include <varargs.h>
-#endif
 #include <limits.h>
 #include <errno.h>
 #include "local.h"
 
 int
-#ifdef _HAVE_STDC
-_DEFUN(_snprintf_r, (ptr, str, size, fmt),
-       struct _reent *ptr _AND
-       char *__restrict str          _AND
-       size_t size        _AND
-       _CONST char *__restrict fmt _DOTS)
-#else
-_snprintf_r(ptr, str, size, fmt, va_alist)
-            struct _reent *ptr;
-            char *str;
-            size_t size;
-            _CONST char *fmt;
-            va_dcl
-#endif
+_snprintf_r (struct _reent *ptr,
+       char *__restrict str,
+       size_t size,
+       const char *__restrict fmt, ...)
 {
   int ret;
   va_list ap;
@@ -58,11 +44,7 @@ _snprintf_r(ptr, str, size, fmt, va_alist)
   f._bf._base = f._p = (unsigned char *) str;
   f._bf._size = f._w = (size > 0 ? size - 1 : 0);
   f._file = -1;  /* No file. */
-#ifdef _HAVE_STDC
   va_start (ap, fmt);
-#else
-  va_start (ap);
-#endif
   ret = _svfprintf_r (ptr, &f, fmt, ap);
   va_end (ap);
   if (ret < EOF)
@@ -72,21 +54,18 @@ _snprintf_r(ptr, str, size, fmt, va_alist)
   return (ret);
 }
 
+#ifdef _NANO_FORMATTED_IO
+int
+_sniprintf_r (struct _reent *, char *, size_t, const char *, ...)
+       _ATTRIBUTE ((__alias__("_snprintf_r")));
+#endif
+
 #ifndef _REENT_ONLY
 
 int
-#ifdef _HAVE_STDC
-_DEFUN(snprintf, (str, size, fmt),
-       char *__restrict str _AND
-       size_t size _AND
-       _CONST char *__restrict fmt _DOTS)
-#else
-snprintf(str, size, fmt, va_alist)
-         char *str;
-         size_t size;
-         _CONST char *fmt;
-         va_dcl
-#endif
+snprintf (char *__restrict str,
+       size_t size,
+       const char *__restrict fmt, ...)
 {
   int ret;
   va_list ap;
@@ -102,11 +81,7 @@ snprintf(str, size, fmt, va_alist)
   f._bf._base = f._p = (unsigned char *) str;
   f._bf._size = f._w = (size > 0 ? size - 1 : 0);
   f._file = -1;  /* No file. */
-#ifdef _HAVE_STDC
   va_start (ap, fmt);
-#else
-  va_start (ap);
-#endif
   ret = _svfprintf_r (ptr, &f, fmt, ap);
   va_end (ap);
   if (ret < EOF)
@@ -116,4 +91,9 @@ snprintf(str, size, fmt, va_alist)
   return (ret);
 }
 
+#ifdef _NANO_FORMATTED_IO
+int
+sniprintf (char *, size_t, const char *, ...)
+       _ATTRIBUTE ((__alias__("snprintf")));
+#endif
 #endif
