@@ -17,59 +17,33 @@
 
 /*
 FUNCTION
-<<scanf>>, <<fscanf>>, <<sscanf>>---scan and format input
+<<sscanf>>, <<fscanf>>, <<scanf>>---scan and format input
 
 INDEX
 	scanf
+INDEX
+	_scanf_r
 INDEX
 	fscanf
+INDEX
+	_fscanf_r
 INDEX
 	sscanf
+INDEX
+	_sscanf_r
 
-ANSI_SYNOPSIS
+SYNOPSIS
         #include <stdio.h>
 
-        int scanf(const char *<[format]> [, <[arg]>, ...]);
-        int fscanf(FILE *<[fd]>, const char *<[format]> [, <[arg]>, ...]);
-        int sscanf(const char *<[str]>, const char *<[format]> 
-                   [, <[arg]>, ...]);
-
-        int _scanf_r(struct _reent *<[ptr]>, const char *<[format]>
-                     [, <[arg]>, ...]);
-        int _fscanf_r(struct _reent *<[ptr]>, FILE *<[fd]>, const char *<[format]>
-                      [, <[arg]>, ...]);
-        int _sscanf_r(struct _reent *<[ptr]>, const char *<[str]>,
-                      const char *<[format]> [, <[arg]>, ...]);
-
-
-TRAD_SYNOPSIS
-	#include <stdio.h>
-
-	int scanf(<[format]> [, <[arg]>, ...])
-	char *<[format]>;
-
-	int fscanf(<[fd]>, <[format]> [, <[arg]>, ...]);
-	FILE *<[fd]>;
-	char *<[format]>;
-
-	int sscanf(<[str]>, <[format]> [, <[arg]>, ...]);
-	char *<[str]>;
-	char *<[format]>;
-
-	int _scanf_r(<[ptr]>, <[format]> [, <[arg]>, ...])
-        struct _reent *<[ptr]>;
-	char *<[format]>;
-
-	int _fscanf_r(<[ptr]>, <[fd]>, <[format]> [, <[arg]>, ...]);
-        struct _reent *<[ptr]>;
-	FILE *<[fd]>;
-	char *<[format]>;
-
-	int _sscanf_r(<[ptr]>, <[str]>, <[format]> [, <[arg]>, ...]);
-        struct _reent *<[ptr]>;
-	char *<[str]>;
-	char *<[format]>;
+        int scanf(const char *restrict <[format]>, ...);
+        int fscanf(FILE *restrict <[fd]>, const char *restrict <[format]>, ...);
+        int sscanf(const char *restrict <[str]>, const char *restrict <[format]>, ...);
 
+        int _scanf_r(struct _reent *<[ptr]>, const char *restrict <[format]>, ...);
+        int _fscanf_r(struct _reent *<[ptr]>, FILE *restrict <[fd]>, 
+                      const char *restrict <[format]>, ...);
+        int _sscanf_r(struct _reent *<[ptr]>, const char *restrict <[str]>,
+                      const char *restrict <[format]>, ...);
 
 DESCRIPTION
         <<scanf>> scans a series of input fields from standard input,
@@ -128,12 +102,14 @@ DESCRIPTION
 
         Each format specification begins with the percent character (<<%>>).
         The other fields are:
-	o+
+	O+
 		o *
+
 		an optional marker; if present, it suppresses interpretation and
         assignment of this input field.
 
         o <[width]>
+
 		an optional maximum field width: a decimal integer,
 		which controls the maximum number of characters that
 		will be read before converting the current input field.  If the
@@ -146,42 +122,115 @@ DESCRIPTION
 		to that character are read, converted, and stored.
 		Then <<scanf>> proceeds to the next format specification.
 
-        o size
-		<<h>>, <<l>>, and <<L>> are optional size characters which
-		override the default way that <<scanf>> interprets the
-		data type of the corresponding argument.
+        o <[size]>
 
+		<<h>>, <<j>>, <<l>>, <<L>>, <<t>>, and <<z>> are optional size
+		characters which override the default way that <<scanf>>
+		interprets the data type of the corresponding argument.
 
-.Modifier   Type(s)
-.   hh      d, i, o, u, x, n  convert input to char,
-.                             store in char object
-.
-.   h       d, i, o, u, x, n  convert input to short,
-.                             store in short object
-.
-.   h       D, I, O, U, X     no effect
-.           e, f, c, s, p
-.
-.   l       d, i, o, u, x, n  convert input to long,
-.                             store in long object
-.
-.   l       e, f, g           convert input to double
-.                             store in a double object
-.
-.   l       D, I, O, U, X     no effect
-.           c, s, p
-.
-.   ll      d, i, o, u, x, n  convert to long long,
-.                             store in long long
-.
-.   L       d, i, o, u, x, n  convert to long long,
-.                             store in long long
-.
-.   L       e, f, g, E, G     convert to long double,
-.                             store in long double
-.
-.   L      all others         no effect
-
+		@multitable @columnfractions 0.18 0.30 0.52
+		@headitem
+		Modifier
+		@tab
+		Type(s)
+		@tab
+		@item
+		hh
+		@tab
+		d, i, o, u, x, n
+		@tab
+		convert input to char, store in char object
+		@item
+		h
+		@tab
+		d, i, o, u, x, n
+		@tab
+		convert input to short, store in short object
+		@item
+		h
+		@tab
+		D, I, O, U, X, e, f, c, s, p
+		@tab
+		no effect
+		@item
+		j
+		@tab
+		d, i, o, u, x, n
+		@tab
+		convert input to intmax_t, store in intmax_t object
+		@item
+		j
+		@tab
+		all others
+		@tab
+		no effect
+		@item
+		l
+		@tab
+		d, i, o, u, x, n
+		@tab
+		convert input to long, store in long object
+		@item
+		l
+		@tab
+		e, f, g
+		@tab
+		convert input to double, store in a double object
+		@item
+		l
+		@tab
+		D, I, O, U, X, c, s, p
+		@tab
+		no effect
+		@item
+		ll
+		@tab
+		d, i, o, u, x, n
+		@tab
+		convert to long long, store in long long object
+		@item
+		L
+		@tab
+		d, i, o, u, x, n
+		@tab
+		convert to long long, store in long long object
+		@item
+		L
+		@tab
+		e, f, g, E, G
+		@tab
+		convert to long double, store in long double object
+		@item
+		L
+		@tab
+		all others
+		@tab
+		no effect
+		@item
+		t
+		@tab
+		d, i, o, u, x, n
+		@tab
+		convert input to ptrdiff_t, store in ptrdiff_t object
+		@item
+		t
+		@tab
+		all others
+		@tab
+		no effect
+		@item
+		z
+		@tab
+		d, i, o, u, x, n
+		@tab
+		convert input to size_t, store in size_t object
+		@item
+		z
+		@tab
+		all others
+		@tab
+		no effect
+		@end multitable
 
         o <[type]>
 
@@ -190,7 +239,7 @@ DESCRIPTION
                 characters:
 
 		o+
-		o  %
+		o %
 		No conversion is done; the percent character (<<%>>) is stored.
 
 		o c
@@ -200,7 +249,7 @@ DESCRIPTION
 		Reads a character string into the array supplied.
 		Corresponding <[arg]>: <<(char arg[])>>.
 
-		o  [<[pattern]>]
+		o [<[pattern]>]
 		Reads a non-empty character string into memory
 		starting at <[arg]>.  This area must be large
 		enough to accept the sequence and a
@@ -224,7 +273,6 @@ DESCRIPTION
 		o u
 		Reads an unsigned decimal integer into the corresponding
 		<[arg]>: <<(unsigned int *arg)>>.
-			
 
 		o U
 		Reads an unsigned decimal integer into the corresponding <[arg]>:
@@ -242,19 +290,19 @@ DESCRIPTION
 		Read a floating-point number into the corresponding <[arg]>:
 		<<(double *arg)>>.
 
-		o  i
+		o i
 		Reads a decimal, octal or hexadecimal integer into the
 		corresponding <[arg]>: <<(int *arg)>>.
 
-		o  I
+		o I
 		Reads a decimal, octal or hexadecimal integer into the
 		corresponding <[arg]>: <<(long *arg)>>.
 
-		o  n
+		o n
 		Stores the number of characters read in the corresponding
 		<[arg]>: <<(int *arg)>>.
 
-		o  p
+		o p
                 Stores a scanned pointer.  ANSI C leaves the details
 		to each implementation; this implementation treats
 		<<%p>> exactly the same as <<%U>>.  Corresponding
@@ -296,7 +344,7 @@ DESCRIPTION
 
 	where objects inclosed in square brackets are optional, and <<ddd>>
 	represents decimal, octal, or hexadecimal digits.
-	o-
+	O-
 
 RETURNS
         <<scanf>> returns the number of input fields successfully
@@ -367,38 +415,14 @@ Supporting OS subroutines required: <<close>>, <<fstat>>, <<isatty>>,
 #include <reent.h>
 #include <stdio.h>
 #include <string.h>
-#ifdef _HAVE_STDC
 #include <stdarg.h>
-#else
-#include <varargs.h>
-#endif
 #include "local.h"
 
-/* | ARGSUSED */
-/*SUPPRESS 590*/
-static _READ_WRITE_RETURN_TYPE
-_DEFUN(eofread, (cookie, buf, len),
-       _PTR cookie _AND
-       char *buf   _AND
-       int len)
-{
-  return 0;
-}
-
 #ifndef _REENT_ONLY 
 
-#ifdef _HAVE_STDC
 int 
-_DEFUN(sscanf, (str, fmt),
-       _CONST char *str _AND
-       _CONST char *fmt _DOTS)
-#else
-int 
-sscanf(str, fmt, va_alist)
-       _CONST char *str;
-       _CONST char *fmt;
-       va_dcl
-#endif
+sscanf (const char *__restrict str,
+       const char * fmt, ...)
 {
   int ret;
   va_list ap;
@@ -407,36 +431,28 @@ sscanf(str, fmt, va_alist)
   f._flags = __SRD | __SSTR;
   f._bf._base = f._p = (unsigned char *) str;
   f._bf._size = f._r = strlen (str);
-  f._read = eofread;
+  f._read = __seofread;
   f._ub._base = NULL;
   f._lb._base = NULL;
   f._file = -1;  /* No file. */
-#ifdef _HAVE_STDC
   va_start (ap, fmt);
-#else
-  va_start (ap);
-#endif
-  ret = __svfscanf_r (_REENT, &f, fmt, ap);
+  ret = __ssvfscanf_r (_REENT, &f, fmt, ap);
   va_end (ap);
   return ret;
 }
 
+#ifdef _NANO_FORMATTED_IO
+int
+siscanf (const char *, const char *, ...)
+       _ATTRIBUTE ((__alias__("sscanf")));
+#endif
+
 #endif /* !_REENT_ONLY */
 
-#ifdef _HAVE_STDC
 int 
-_DEFUN(_sscanf_r, (ptr, str, fmt), 
-       struct _reent *ptr _AND
-       _CONST char *str   _AND
-       _CONST char *fmt _DOTS)
-#else
-int 
-_sscanf_r(ptr, str, fmt, va_alist)
-          struct _reent *ptr;
-          _CONST char *str;
-          _CONST char *fmt;
-          va_dcl
-#endif
+_sscanf_r (struct _reent *ptr,
+       const char *__restrict str,
+       const char *__restrict fmt, ...)
 {
   int ret;
   va_list ap;
@@ -445,16 +461,18 @@ _sscanf_r(ptr, str, fmt, va_alist)
   f._flags = __SRD | __SSTR;
   f._bf._base = f._p = (unsigned char *) str;
   f._bf._size = f._r = strlen (str);
-  f._read = eofread;
+  f._read = __seofread;
   f._ub._base = NULL;
   f._lb._base = NULL;
   f._file = -1;  /* No file. */
-#ifdef _HAVE_STDC
   va_start (ap, fmt);
-#else
-  va_start (ap);
-#endif
-  ret = __svfscanf_r (ptr, &f, fmt, ap);
+  ret = __ssvfscanf_r (ptr, &f, fmt, ap);
   va_end (ap);
   return ret;
 }
+
+#ifdef _NANO_FORMATTED_IO
+int
+_siscanf_r (struct _reent *, const char *, const char *, ...)
+       _ATTRIBUTE ((__alias__("_sscanf_r")));
+#endif
